from cmutils.config_loaders import IniConfigLoader
from peewee import *
import json
from pymongo import MongoClient
from playhouse.shortcuts import ReconnectMixin


database_proxy = Proxy()


class ReconnectMySQLDatabase(ReconnectMixin, MySQLDatabase):
    pass


def init_mongo_db(config_path=None):
    config = IniConfigLoader(config_path)
    config = config.load()['mongo_db']

    host=config.get('host')
    user=config.get('user')
    password=config.get('password')
    port=config.get('port')
    mongo_uri = f"mongodb://{user}:{password}@{host}:{port}/?authSource=admin"
    return MongoClient(mongo_uri)


def init_db(config_path=None):
    config = IniConfigLoader(config_path)
    config = config.load()['store_db']

    db_connection = ReconnectMySQLDatabase(
        host=config.get('host'),
        user=config.get('user'),
        password=config.get('password'),
        database=config.get('database')
    )
    database_proxy.initialize(db_connection)



class BaseModel(Model):
    def __str__(self):
        r = {}
        for k in self._data.keys():
            try:
                r[k] = str(getattr(self, k))
            except:
                r[k] = json.dumps(getattr(self, k))
        return str(r)

    class Meta:
        database = database_proxy

class Store(BaseModel):
    name = CharField()
    website = CharField()
    site_name = CharField()
    api_endpoint = CharField()
    api_key = CharField()
    api_password = CharField()
    api_version = CharField()
    api_token = CharField()
    merchant_id = CharField()
    service_account = TextField()
    spreadsheet_id = CharField()
    general_spreadsheet_id = CharField()
    last_order_id = IntegerField()
    last_order_date = DateTimeField()
    location_id = IntegerField()
    status = IntegerField()
    roi = DecimalField()
    ads_cost = DecimalField()
    transfer_cost = DecimalField()
    reset_all_prices = IntegerField()
    report_token = CharField()
    platform = CharField()
    currencies = CharField()
    locale = CharField()
    paypal_client = CharField()
    paypal_secret = CharField()
    sheet_name_format = CharField()
    use_site_image = IntegerField()
    important = IntegerField()
    enable_expedited = IntegerField()
    gads_credential = CharField()
    gads_customer_id = CharField()
    region = CharField()

    @staticmethod
    def get_store(name):
        stores = Store.select().where((Store.site_name == name) | (Store.name == name))
        if len(stores) == 0:
            return None
            # raise Exception("no store with name %s found" % name)
        for store in stores:
            return store

    class Meta:
        db_table = 'stores'


class SpreeProduct(BaseModel):
    id = IntegerField()
    name = CharField()
    description = TextField()
    returnable = BooleanField()
    edd = CharField()
    unique_identifier = CharField()
    discontinue_on = DateTimeField()
    deleted_at = DateTimeField()
    updated_at = DateTimeField()
    shipping_category_id = IntegerField()

    class Meta:
        db_table = 'spree_products'


class SpreeProductSource(BaseModel):
    id = CharField()
    product_id = IntegerField()
    source = CharField()
    source_product_id = CharField()
    source_product_url = CharField()

    class Meta:
        db_table = 'spree_product_sources'
